@@ -1,4 +1,5 @@
 use super::dict::*;
+use super::jyutping::*;
 use super::parse::*;
 use super::rich_dict;
 use lip::assert_succeed;
@@ -44,6 +45,81 @@ fn normal_word(string: &'static str) -> rich_dict::Word {
 }
 
 #[cfg(test)]
+#[test]
+fn test_parse_line() {
+    // Need to delete a single space before '#' if previous character is a CJK
+    // Need to delete a single space following a link (unconditional)
+    assert_succeed(
+        parse_line("yue"),
+        "有個詞叫 #你好 係好常用嘅。",
+        vec![text("有個詞叫"), link("你好"), text("係好常用嘅。")],
+    );
+
+    assert_succeed(
+        parse_line("eng"),
+        "particle #喇 laa3 before the particles #喎  wo3, #噃 bo3 or #可 ho2",
+        vec![
+            text("particle "),
+            link("喇"),
+            text(" laa3 before the particles "),
+            link("喎"),
+            text(" wo3, "),
+            link("噃"),
+            text(" bo3 or "),
+            link("可"),
+            text(" ho2"),
+        ],
+    );
+
+    assert_succeed(
+        parse_line("eng"),
+        "less common than #等等 dang2 dang2",
+        vec![
+            text("less common than "),
+            link("等等"),
+            text(" dang2 dang2"),
+        ],
+    );
+
+    assert_succeed(
+        parse_line("eng"),
+        "equavalent to #現 (jin6)",
+        vec![text("equavalent to "), link("現"), text(" (jin6)")],
+    );
+
+    assert_succeed(
+        parse_line("即 #未雨綢繆"),
+        "即 #未雨綢繆",
+        vec![text("即"), link("未雨綢繆")],
+    );
+
+    assert_succeed(
+        parse_line("yue"),
+        "#質素 同 #數量 嘅合稱",
+        vec![link("質素"), text("同"), link("數量"), text("嘅合稱")],
+    );
+
+    assert_succeed(
+        parse_line("eng"),
+        "as a variant of #1 with a weaker",
+        vec![text("as a variant of "), link("1"), text(" with a weaker")],
+    );
+
+    assert_succeed(
+        parse_line("yue"),
+        "eg. #咁 gam3, #勁 ging6 or #最 zeoi3",
+        vec![
+            text("eg. "),
+            link("咁"),
+            text(" gam3, "),
+            link("勁"),
+            text(" ging6 or "),
+            link("最"),
+            text(" zeoi3"),
+        ],
+    );
+}
+
 #[test]
 fn test_parse_clause() {
     assert_succeed(
@@ -84,6 +160,62 @@ fn test_parse_pr_line() {
         ),
     );
     assert_succeed(parse_pr_line("yue"), "yue:條八婆好扮嘢㗎，連嗌個叉飯都要講英文。 (tiu4 baat3 po4 hou2 baan6 je5 gaa3, lin4 aai3 go3 caa1 faan6 dou1 jiu3 gong2 jing1 man2.)", (simple_line("條八婆好扮嘢㗎，連嗌個叉飯都要講英文。"), Some("tiu4 baat3 po4 hou2 baan6 je5 gaa3, lin4 aai3 go3 caa1 faan6 dou1 jiu3 gong2 jing1 man2.".to_string())));
+
+    // Test parentheses detection
+    // i.e. check whether the content in the parentheses
+    // is actually jyutping
+    assert_succeed(
+        parse_pr_line("yue"),
+        "yue:p分別為聖。(約翰福音 17:19)",
+        (simple_line("p分別為聖。(約翰福音 17:19)"), None),
+    );
+
+    assert_succeed(
+        parse_pr_line("yue"),
+        "yue:#質素 同 #數量 嘅合稱 (this is an side note with jyut6 ping3, not a pr)",
+        (
+            vec![
+                link("質素"),
+                text("同"),
+                link("數量"),
+                text("嘅合稱 (this is an side note with jyut6 ping3, not a pr)"),
+            ],
+            None,
+        ),
+    );
+
+    assert_succeed(
+        parse_pr_line("eng"),
+        "eng:particle #喇 laa3 before the particles #喎  wo3, #噃 bo3 or #可 ho2(this is an side note, not a pr)",
+        (vec![
+            text("particle "),
+            link("喇"),
+            text(" laa3 before the particles "),
+            link("喎"),
+            text(" wo3, "),
+            link("噃"),
+            text(" bo3 or "),
+            link("可"),
+            text(" ho2(this is an side note, not a pr)"),
+        ], None),
+    );
+
+    assert_succeed(
+        parse_pr_line("yue"),
+        "yue:eg. #咁 gam3, #勁 ging6 or #最 zeoi3(this is an side note, not a pr)",
+        (
+            vec![
+                text("eg. "),
+                link("咁"),
+                text(" gam3, "),
+                link("勁"),
+                text(" ging6 or "),
+                link("最"),
+                text(" zeoi3(this is an side note, not a pr)"),
+            ],
+            None,
+        ),
+    );
 }
 
 #[test]
@@ -795,6 +927,48 @@ fn test_match_ruby() {
         ]
     );
 
+    // typo in jyutping "能" nan4 should be nang4
+    assert_eq!(
+        match_ruby(
+            &vec!["懷春".into()],
+            &vec![text("能曾經有懷春嘅日子。")],
+            &vec!["nan4", "cang4", "ging1", "jau5", "waai4", "ceon1", "ge3", "jat6", "zi2"]
+        ),
+        vec![
+            Word(normal_word("能"), vec!["nan4".into()]),
+            Word(normal_word("曾"), vec!["cang4".into()]),
+            Word(normal_word("經"), vec!["ging1".into()]),
+            Word(normal_word("有"), vec!["jau5".into()]),
+            Word(bold_word("懷"), vec!["waai4".into()]),
+            Word(bold_word("春"), vec!["ceon1".into()]),
+            Word(normal_word("嘅"), vec!["ge3".into()]),
+            Word(normal_word("日"), vec!["jat6".into()]),
+            Word(normal_word("子"), vec!["zi2".into()]),
+            Punc("。".into())
+        ]
+    );
+
+    // typo in jyutping "曾" can4 should be cang4
+    assert_eq!(
+        match_ruby(
+            &vec!["懷春".into()],
+            &vec![text("能曾經有懷春嘅日子。")],
+            &vec!["nang4", "can4", "ging1", "jau5", "waai4", "ceon1", "ge3", "jat6", "zi2"]
+        ),
+        vec![
+            Word(normal_word("能"), vec!["nang4".into()]),
+            Word(normal_word("曾"), vec!["can4".into()]),
+            Word(normal_word("經"), vec!["ging1".into()]),
+            Word(normal_word("有"), vec!["jau5".into()]),
+            Word(bold_word("懷"), vec!["waai4".into()]),
+            Word(bold_word("春"), vec!["ceon1".into()]),
+            Word(normal_word("嘅"), vec!["ge3".into()]),
+            Word(normal_word("日"), vec!["jat6".into()]),
+            Word(normal_word("子"), vec!["zi2".into()]),
+            Punc("。".into())
+        ]
+    );
+
     // typo in jyutping "能" nan4 should be nang4, "曾" can4 should be cang4
     assert_eq!(
         match_ruby(
@@ -869,6 +1043,149 @@ fn test_match_ruby() {
             Word(normal_word("萬"), vec!["maan6".into()])
         ]
     );
+
+    // one Chinese character two prs
+    assert_eq!(
+        match_ruby(
+            &vec!["卅".into()],
+            &vec![text("卅幾")],
+            &vec!["saa1", "aa6", "gei2"]
+        ),
+        vec![
+            Word(bold_word("卅"), vec!["saa1".into(), "aa6".into()]),
+            Word(normal_word("幾"), vec!["gei2".into()]),
+        ]
+    );
+
+    assert_eq!(
+        match_ruby(
+            &vec!["卅".into()],
+            &vec![link("年卅晚")],
+            &vec!["nin4", "saa1", "aa6", "maan5"]
+        ),
+        vec![LinkedWord(vec![
+            (normal_word("年"), vec!["nin4".into()]),
+            (bold_word("卅"), vec!["saa1".into(), "aa6".into()]),
+            (normal_word("晚"), vec!["maan5".into()])
+        ])]
+    );
+    assert_eq!(
+        match_ruby(
+            &vec!["卌".into()],
+            &vec![text("卌二")],
+            &vec!["se3", "aa6", "ji6"]
+        ),
+        vec![
+            Word(bold_word("卌"), vec!["se3".into(), "aa6".into()]),
+            Word(normal_word("二"), vec!["ji6".into()]),
+        ]
+    );
+
+    // Q&A-style without ruby annotations for 甲 and 乙
+    assert_eq!(
+        match_ruby(
+            &vec!["大家噉話"],
+            &vec![text("甲：「身體健康！」乙：「大家噉話！」")],
+            &vec!["san1", "tai2", "gin6", "hong1", "daai6", "gaa1", "gam2", "waa6"]
+        ),
+        vec![
+            Word(normal_word("甲"), vec![]),
+            Punc("：".into()),
+            Punc("「".into()),
+            Word(normal_word("身"), vec!["san1".into()]),
+            Word(normal_word("體"), vec!["tai2".into()]),
+            Word(normal_word("健"), vec!["gin6".into()]),
+            Word(normal_word("康"), vec!["hong1".into()]),
+            Punc("！".into()),
+            Punc("」".into()),
+            Word(normal_word("乙"), vec![]),
+            Punc("：".into()),
+            Punc("「".into()),
+            Word(bold_word("大"), vec!["daai6".into()],),
+            Word(bold_word("家"), vec!["gaa1".into()],),
+            Word(bold_word("噉"), vec!["gam2".into()],),
+            Word(bold_word("話"), vec!["waa6".into()],),
+            Punc("！".into()),
+            Punc("」".into()),
+        ]
+    );
+
+    // Q&A-style with ruby annotations for 甲 and 乙
+    assert_eq!(
+        match_ruby(
+            &vec!["囉"],
+            &vec![
+                text("甲：「"),
+                link("方丈"),
+                text("真係好小器呀！」乙：「係囉！」")
+            ],
+            &vec![
+                "gaap3", "fong1", "zoeng6", "zan1", "hai6", "hou2", "siu2", "hei3", "aa3", "jyut6",
+                "hai6", "lo1"
+            ]
+        ),
+        vec![
+            Word(normal_word("甲"), vec!["gaap3".into()]),
+            Punc("：".into()),
+            Punc("「".into()),
+            LinkedWord(vec![
+                (normal_word("方"), vec!["fong1".into()]),
+                (normal_word("丈"), vec!["zoeng6".into()])
+            ]),
+            Word(normal_word("真"), vec!["zan1".into()]),
+            Word(normal_word("係"), vec!["hai6".into()]),
+            Word(normal_word("好"), vec!["hou2".into()]),
+            Word(normal_word("小"), vec!["siu2".into()]),
+            Word(normal_word("器"), vec!["hei3".into()]),
+            Word(normal_word("呀"), vec!["aa3".into()]),
+            Punc("！".into()),
+            Punc("」".into()),
+            Word(normal_word("乙"), vec!["jyut6".into()]),
+            Punc("：".into()),
+            Punc("「".into()),
+            Word(normal_word("係"), vec!["hai6".into()],),
+            Word(bold_word("囉"), vec!["lo1".into()],),
+            Punc("！".into()),
+            Punc("」".into()),
+        ]
+    );
+
+    // Q&A-style without ruby annotations for 甲 and 乙
+    assert_eq!(
+        match_ruby(
+            &vec!["囉"],
+            &vec![
+                text("甲：「"),
+                link("方丈"),
+                text("真係好小器呀！」乙：「係囉！」")
+            ],
+            &vec!["fong1", "zoeng6", "zan1", "hai6", "hou2", "siu2", "hei3", "aa3", "hai6", "lo1"]
+        ),
+        vec![
+            Word(normal_word("甲"), vec![]),
+            Punc("：".into()),
+            Punc("「".into()),
+            LinkedWord(vec![
+                (normal_word("方"), vec!["fong1".into()]),
+                (normal_word("丈"), vec!["zoeng6".into()])
+            ]),
+            Word(normal_word("真"), vec!["zan1".into()]),
+            Word(normal_word("係"), vec!["hai6".into()]),
+            Word(normal_word("好"), vec!["hou2".into()]),
+            Word(normal_word("小"), vec!["siu2".into()]),
+            Word(normal_word("器"), vec!["hei3".into()]),
+            Word(normal_word("呀"), vec!["aa3".into()]),
+            Punc("！".into()),
+            Punc("」".into()),
+            Word(normal_word("乙"), vec![]),
+            Punc("：".into()),
+            Punc("「".into()),
+            Word(normal_word("係"), vec!["hai6".into()],),
+            Word(bold_word("囉"), vec!["lo1".into()],),
+            Punc("！".into()),
+            Punc("」".into()),
+        ]
+    );
 }
 
 #[test]
